@@ -2,8 +2,9 @@
 #  Copyright © 2022 Ingram Micro Inc. All rights reserved.
 #
 
-from cachetools import LFUCache
+from threading import Lock
 
+from cachetools import LFUCache
 from lark import Lark
 from lark.exceptions import LarkError
 
@@ -16,17 +17,23 @@ class RQLLarkParser(Lark):
         super(RQLLarkParser, self).__init__(*args, **kwargs)
 
         self._cache = LFUCache(maxsize=1000)
+        self._lock = Lock()
 
     def parse_query(self, query):
         cache_key = hash(query)
-        if cache_key in self._cache:
-            return self._cache[cache_key]
+
         try:
-            rql_ast = self.parse(query)
-            self._cache[cache_key] = rql_ast
-            return rql_ast
-        except LarkError:
-            raise RQLFilterParsingError()
+            return self._cache[cache_key]
+        except KeyError:
+
+            try:
+                rql_ast = self.parse(query)
+                with self._lock:
+                    self._cache[cache_key] = rql_ast
+
+                return rql_ast
+            except LarkError:
+                raise RQLFilterParsingError()
 
 
 RQLParser = RQLLarkParser(RQL_GRAMMAR, parser='lalr', start='start')
